/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OEone Calendar Code, released October 31st, 2001.
 *
 * The Initial Developer of the Original Code is
 * OEone Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Garth Smedley <garths@oeone.com>
 *                 Mike Potter <mikep@oeone.com>
 *                 Chris Charabaruk <coldacid@meldstar.com>
 *                 Colin Phillips <colinp@oeone.com>
 *                 ArentJan Banck <ajbanck@planet.nl>
 *                 Curtis Jewell <csjewell@mail.freeshell.org>
 *                 Eric Belhaire <eric.belhaire@ief.u-psud.fr>
 *                 Mark Swaffer <swaff@fudo.org>
 *                 Michael Buettner <michael.buettner@sun.com>
 *                 Philipp Kewisch <mozilla@kewis.ch>
 *                 Matthew Mecca <matthew.mecca@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * Called when the window is loaded to set up the unifinder-todo.
 */
function prepareCalendarToDoUnifinder() {
    if (isSunbird()) {
        document.getElementById("todo-label").removeAttribute("collapsed");
    }

    // add listener to update the date filters
    getViewDeck().addEventListener("dayselect", updateCalendarToDoUnifinder, false);

    updateCalendarToDoUnifinder();
}

/**
 * Updates the applied filter and show completed view of the unifinder todo.
 *
 * @param aFilter        The filter name to set.
 */
function updateCalendarToDoUnifinder(aFilter) {
    // Set up hiding completed tasks for the unifinder-todo tree
    let showCompleted = document.getElementById("show-completed-checkbox").checked;
    let tree = document.getElementById("unifinder-todo-tree");
    let oldFilter = document.getElementById("unifinder-todo-filter-broadcaster").getAttribute("value");
    let filter = oldFilter;

    // This function acts as an event listener, in which case we get the Event as the 
    // parameter instead of a filter.
    if (aFilter && !(aFilter instanceof Event)) {
        filter = aFilter;
    }

    if (filter && (filter != oldFilter)) {
        document.getElementById("unifinder-todo-filter-broadcaster").setAttribute("value", aFilter);
    }

    // update the filter
    tree.showCompleted = showCompleted;
    tree.updateFilter(filter);
}

/**
 * Called when the window is unloaded to clean up the unifinder-todo.
 */
function finishCalendarToDoUnifinder() {
    // remove listeners
    getViewDeck().removeEventListener("dayselect", updateCalendarToDoUnifinder, false);
}
