/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Oracle Corporation code
 *
 * The Initial Developer of the Original Code is
 * Oracle Corporation
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Stuart Parmenter <stuart.parmenter@oracle.com>
 *   Simon Paquet <bugzilla@babylonsounds.com>
 *   Daniel Boelzle <daniel.boelzle@sun.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK *****/

Components.utils.import("resource://calendar/modules/calUtils.jsm");

/**
 * Global Object to hold methods for the timezones dialog.
 */
var gTimezonesPane = {
    /**
     * Initialize the timezones pref pane. Sets up dialog controls to match the
     * values set in prefs.
     */
    init: function gTP_init() {
        var tzMenuList = document.getElementById("calendar-timezone-menulist");
        var tzMenuPopup = document.getElementById("calendar-timezone-menupopup");

        var tzService = cal.getTimezoneService();
        var enumerator = tzService.timezoneIds;
        var tzids = {};
        var displayNames = [];
        // don't rely on what order the timezone-service gives you
        while (enumerator.hasMore()) {
            var tz = tzService.getTimezone(enumerator.getNext());
            if (tz && !tz.isFloating && !tz.isUTC) {
                var displayName = tz.displayName;
                displayNames.push(displayName);
                tzids[displayName] = tz.tzid;
            }
        }
        // the display names need to be sorted
        displayNames.sort(String.localeCompare);
        for (var i = 0; i < displayNames.length; ++i) {
            var displayName = displayNames[i];
            addMenuItem(tzMenuPopup, displayName, tzids[displayName]);
        }

        var prefValue = document.getElementById("calendar-timezone-local").value;
        if (!prefValue) {
            prefValue = calendarDefaultTimezone().tzid;
        }
        tzMenuList.value = prefValue;
    }
};
